/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.container;

import java.util.Set;
import org.objectweb.jonas.service.Service;
import org.objectweb.jonas_ejb.container.Container;
import javax.naming.Context;

/**
 * EJB Service interface.
 */
public interface EJBService extends Service {

    /**
     * Create a JOnAS Container for all beans that are described in a .xml file 
     * or belong to .jar file.
     * @param ctx JNDI contect in which are found the container configuration.
     */
    public Container createContainer(Context ctx) throws Exception;

    /**
     * Get the Container by its file name (.xml or .jar)
     */
    public Container getContainer(String fileName);

    /**
     * Remove a  JOnAS Container.
     * @param cont JOnAS container to remove.
     * @return Container object
     */
    public void removeContainer(Container cont);

    /**
     * List of all JOnAS Containers
     * @return an array of Container objects
     */
    public Container[] listContainers();

    /**
     * Synchronized all entity bean containers
     * @param passivate passivate instances after synchronization.
     */
    public void syncAllEntities(boolean passivate);
}

