/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Miroslav halas
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jetty;

import javax.naming.Context;
import javax.naming.NamingException;
import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.StringTokenizer;

import org.objectweb.jonas.common.Log;
import org.objectweb.jonas.service.ServiceException;
import org.objectweb.jonas.service.AbsDynamicServiceImpl;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

import org.mortbay.jetty.*;
import org.mortbay.util.*;

/**
 * Jetty service implementation for Jetty 4
 * This class modeled based on EmbeddedTomcatImpl33.java
 * @author Miroslav Halas
 */
public class EmbeddedJettyImpl4 extends    AbsDynamicServiceImpl 
    implements JettyService 
{
    /**
     * Private logger for this class.
    */
    static private Logger logger = null;

    /**
    * Embedded Jetty server.
    */
    private Server m_srvJetty;
   
    private String argsforTomcat[]= null;

   /**
    * Name of the variable specifyin configuration file for Jetty
    */
    private static final String JETTY_CONFIG = "config";

    /**
    * Initialize Jetty server.
    *
    * @param ctx
    */
    public void doInit(Context ctx) {
        if (logger == null) {
            logger = Log.getLogger(Log.JONAS_TOMCAT_PREFIX);
        }
      
        logger.log(BasicLevel.DEBUG, "entering");
      
        String strConfig = null;
      
        try {
            strConfig = (String)ctx.lookup(JETTY_CONFIG);          
        } catch (NamingException neExc) {
            // by default we take jetty.xml under JONAS_ROOT/config
            String jonasroot = System.getProperties().getProperty("install.root");
            strConfig = jonasroot+"/config/jetty.xml";
        }
        
        // First create the server
        m_srvJetty = new Server();
      
        // Now configure it in separate step in case something goes wrong
        try {
            m_srvJetty.configure(strConfig);
        } catch (IOException ioeExc) {
            logger.log(BasicLevel.LEVEL_ERROR, "Cannot configure Jetty server using configuration file " 
                       + strConfig, ioeExc);
        }      
    }

    /**
    * Start Jetty service.
    *
    * @throws ServiceException
    */
    public void run() throws ServiceException {
        logger.log(BasicLevel.DEBUG, "entering");
        try {
            m_srvJetty.start();
        } catch(MultiException meExc) {
            throw new ServiceException("Error while starting the Jetty server", 
                                       meExc);
        }
    }

    /**
    * Stop Jetty service
    *
    * @throws ServiceException
    */
    public void doStop() throws ServiceException {
        logger.log(BasicLevel.DEBUG, "entering");
        if (isStarted()){
            try {
                m_srvJetty.stop();
            } catch (InterruptedException ieExc) {
                throw new ServiceException("Error while stopping the Jetty server", ieExc);
            } finally {
                m_srvJetty.destroy();
            }
        }
    }
}

