/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jmx.vjadmin.servlet;

// servlet imports
import javax.servlet.*;
import javax.servlet.http.*;

// standard imports
import java.util.Date;
import java.util.Properties;
import java.util.Iterator;
import java.util.Enumeration;
import java.util.Vector;

// velocity imports
import org.apache.velocity.Template;
import org.apache.velocity.context.Context;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.servlet.VelocityServlet;
import org.apache.velocity.app.Velocity;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.apache.velocity.exception.ParseErrorException;

// JOnAS JMX imports
import org.objectweb.jonas.jmx.*;
// JMX
import javax.management.*;

// Jadmin imports
import org.objectweb.jonas.jmx.vjadmin.*;

// monolog imports
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

// JOnAS log imports
import org.objectweb.jonas.common.Log;

public class AllMbeansServiceServlet extends JadminServlet {

    /**
     *
     *
     *  @param ctx a Velocity Context object to be filled with
     *             data.  Will be used for rendering this 
     *             template
     *  @return Template to be used for request
     */
    public Template handleRequest( HttpServletRequest request, 
				   HttpServletResponse response, 
				   Context context ) {
	// Exception treatement
	String alertException = null;

 	// Get Date
	String date = java.text.DateFormat.getInstance().format(new Date());
	context.put("date", date);

	// Get the name of the current JOnAS server
	boolean existsServer = JonasManagementRepr.isRegistered(JonasObjectName.ejbServer());
	context.put("existsServer", new Boolean(existsServer));
	if (existsServer) {
	    String serverName = (String)JonasManagementRepr.getAttribute(JonasObjectName.ejbServer(), "JonasName");
	    context.put("serverName", serverName);
	}
	String mbeansError = "";
	try {
	    // Getting all ObjectNames:
	    java.util.Set allObjectNames = ConnectorFactory.getRMIConnector().queryNames(null, null);
	    // Parsing the ObjectNames
	    Vector managedObjects = new Vector();
	    for (Iterator i=allObjectNames.iterator() ; i.hasNext() ; ) {
		ObjectName currentObjectName = (ObjectName)i.next();
		MBeanInfo currentMBeanInfo = ConnectorFactory.getRMIConnector().getMBeanInfo(currentObjectName);
		// Getting all MBeanAttributeInfo:
		MBeanAttributeInfo[] currentMBeanAttributeInfo = currentMBeanInfo.getAttributes();
		// Parsing all MBeanAttributeInfo
		Vector infos = new Vector();
		for (int j=0 ; j<currentMBeanAttributeInfo.length ; j++) {
		    String type = currentMBeanAttributeInfo[j].getType();	
		    String name = currentMBeanAttributeInfo[j].getName();
		    Object detail = ConnectorFactory.getRMIConnector().getAttribute(currentObjectName, name);
		    ManagedAttributeInfo info = new ManagedAttributeInfo (type, 
									  currentMBeanAttributeInfo[j].isWritable(), 
									  currentMBeanAttributeInfo[j].isReadable(),
									  name,
									  detail);
		    infos.add(info);
		}
		
		managedObjects.add(new ManagedObject((String)currentMBeanInfo.getClassName(), currentObjectName, infos));
	    }
	    context.put("managedObjects", managedObjects);
	} catch (Exception e) {
	    mbeansError = "Error while looking for MBeans: " + e.toString();
	}
	context.put("mbeansError", mbeansError);

	// Return template
	Template template = null;
	try {
	    template = getTemplate("ALLMBEANS.vm"); 
	} catch (ParseErrorException pee ) {
            logger.log(BasicLevel.ERROR, "Parse error for template " + pee);
        } catch( ResourceNotFoundException rnfe) {
            logger.log(BasicLevel.ERROR, "Template not found " + rnfe);
        } catch( Exception e) {
            logger.log(BasicLevel.ERROR, "Error " + e);
        }

	return template;
    }
}
