/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jmx.vjadmin.servlet;

import java.util.Date;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.context.Context;
import org.apache.velocity.servlet.VelocityServlet;
import org.objectweb.jonas.common.Log;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * Servlet inherited by all other Jadmin servlets.
 * @author Philippe Durieux, Adriana Danes (JOnAS team)
 */
public abstract class JadminServlet extends VelocityServlet {
    static Logger logger = null;

    /**
     * Performs initialization of this servlet. 
     * Called by the servlet container on loading.
     */
    public void init(ServletConfig config ) throws ServletException {
	super.init(config);
	logger = Log.getLogger("org.objectweb.jonas.jmx.vjadmin.servlet");
    }

    /** 
     * Creates a VelocityContext.
     */
    protected Context createContext(HttpServletRequest req, HttpServletResponse res) { 
	HttpSession session = req.getSession();
	// Try to get the initial VelocityContext
	VelocityContext ictx = (VelocityContext) session.getAttribute("initialContext");
	if (ictx == null) {
	    // initial context lost, recreate it
	    ictx = InitVeloContext.doIt(session);
	    logger.log(BasicLevel.DEBUG, "Initial VelocityContext lost. Recreate a new one");
	}
	VelocityContext ctx = new VelocityContext(ictx);
	logger.log(BasicLevel.DEBUG, "Create a new VelocityContext containing the initial one");
	return ctx;
    }

    /**
     * get the current date
     */
    protected void getDate(Context ctx) {
 	// Get current Date and put it in ctx
	ctx.put("date", java.text.DateFormat.getInstance().format(new Date()));
    }
}
