/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.clients.entity;

import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import junit.framework.Test;
import junit.framework.TestSuite;
import org.objectweb.jonas.jtests.beans.ebasic.SimpleHome;
import org.objectweb.jonas.jtests.beans.ebasic.Simple;
import java.util.Enumeration;

/**
 * This set of test are basic tests on CMP entity home interface.
 * Only specific tests to CMP are here, all tests common to CMP and BMP
 * are in the inherited class A_BasicHomeInterface.
 * @author Philippe Coq, Philippe Durieux (jonas team)
 */
public class F_BasicHomeInterfaceEC extends A_BasicHomeInterface {

    private static String BEAN_HOME = "ebasicSimpleECHome";
    protected static SimpleHome home = null;

    public F_BasicHomeInterfaceEC(String name) {
	super(name);
    }
  
    // --------------------------------------------------------------------
    // tests on clause where for CMP finder methods
    // These tests are specific to CMP 
    // 
    // ---------------------------------------------------------------------


    /**
     * testFinderBetween  verify a finder method that return a Enumeration
     * whith a where clause :
     *  where ?1 < c_info and c_info < ?2 and ?1 < c_numtest and c_numtest < ?2
     * the purpose  of this test is to verify the correct interpretation of parameters
     * ?1 ?2
     *
     * Must return [pk8,pk9]
     * this is a test for CMP
     *
     */
    public void testFinderBetween() throws Exception {
        Simple entity = null;
        Enumeration eList = null;
        eList = getHome().findInfo_beetwen(10, 60);
        int nb = 0;
        while (eList.hasMoreElements()) {
            entity = (Simple) javax.rmi.PortableRemoteObject.narrow(eList.nextElement(), Simple.class);
            nb++;
            int f1 = entity.getInfo(); 
            assertTrue((f1 > 10) &&  (f1 < 60));
        }
        assertEquals(2, nb);
    }

    /**
     * testFinderEqualOne  verify a finder method that return a Enumeration
     * whith a where clause :
     *   where c_info = ?1 and c_numtest = ?1
     * the object of this test is to verify the correct interpretation of parameters
     * ?1 ?2
     *
     * must return [pk10,pk11]
     *
     * this is a test for CMP
     *
     */
    public void testFinderEqualOne() throws Exception {
        Simple entity = null;
        Enumeration eList = null;
        eList = getHome().findEqualOne(80);
        int nb = 0;
        while (eList.hasMoreElements()) {
            entity = (Simple) javax.rmi.PortableRemoteObject.narrow(eList.nextElement(), Simple.class);
            nb++;
            int f1 = entity.getInfo(); 
            int f2 = entity.getNumTest();
            assertTrue((f1 == 80 ) && (f2 == 80 )) ;
        }
        assertEquals(2, nb);
    }

    /**
     * testFinderEqualTwo  verify a finder method that return a Enumeration
     * whith a where clause :
     *   where c_info = ? and c_numtest = ?
     * the object of this test is to verify the correct interpretation of parameters
     * ?1 ?2
     *
     * must return [pk13]
     * this is a test for CMP
     *
     */

    public void testFinderEqualTwo() throws Exception {
        Simple entity = null;
        Enumeration eList = null;
        eList = getHome().findEqualTwo(100,90);
        int nb = 0;
        while (eList.hasMoreElements()) {
            entity = (Simple) javax.rmi.PortableRemoteObject.narrow(eList.nextElement(), Simple.class);
            nb++;
            int f1 = entity.getInfo(); 
            int f2 = entity.getNumTest();
            assertTrue((f1 == 100 ) && (f2 == 90 )) ;
        }
        assertEquals(1, nb);
    }

    public SimpleHome getHome() {
	if (home == null) {
	    try {
		home = (SimpleHome) PortableRemoteObject.narrow(ictx.lookup(BEAN_HOME), SimpleHome.class);
	    } catch (NamingException e) {
		fail("Cannot get bean home");
	    }
	}
	return home;
    }

    /**
     * init environment:
     * - load beans
     * - create/init database for entities.
     */
    protected void setUp() {
	super.setUp();
    }

    /**
     * This suite is all CMP test cases
     */
    public static Test suite() {
	return new TestSuite(F_BasicHomeInterfaceEC.class);
    }

    public static void main (String args[]) {
	String testtorun = null;
	// Get args
	for (int argn = 0; argn < args.length; argn++) {
	    String s_arg = args[argn];
	    Integer i_arg;
	    if (s_arg.equals("-n")) {
		testtorun = args[++argn];
	    }
	}
	if (testtorun == null) {
	    junit.textui.TestRunner.run(suite());
	} else {
	    junit.textui.TestRunner.run(new F_BasicHomeInterfaceEC(testtorun));
	}
    }
}
