/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.clients.entity;

import java.rmi.RemoteException;
import java.util.Collection;
import javax.ejb.CreateException;
import javax.ejb.DuplicateKeyException;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import junit.framework.Assert;
import junit.framework.Test;
import junit.framework.TestSuite;

import org.objectweb.jonas.jtests.beans.ebasic.Account;
import org.objectweb.jonas.jtests.beans.ebasic.AccountHome;
import org.objectweb.jonas.jtests.beans.ebasic.Person;
import org.objectweb.jonas.jtests.beans.ebasic.PersonHome;
import org.objectweb.jonas.jtests.util.JTestCase;

/**
 * These are tests about the different cases of the primary key type:
 * - primary key that maps to a single field in the entity bean class (java.lang.Integer),
 * - unknown primary key class at the bean development phase,
 *   (defered primary key type specification to the deployment phase).
 *   Note that the compilation of this kind of bean is already a good test !!
 * The case of the primary key maps to multiple fields in the entity bean class
 * is already tested in the "inherit" test.
 * Beans used: ebasic/Account, 
 * @author Helene Joanin (jonas team)
 */
public class F_VariousPK extends JTestCase {

    private static String BEAN_HOME_ACCOUNT = "ebasicAccountECHome";
    protected static AccountHome ahome = null;
    private static String BEAN_HOME_PERSON = "ebasicPersonECHome";
    protected static PersonHome phome = null;

    public F_VariousPK(String name) {
	super(name);
    }

    protected void setUp() {
	super.setUp();
	if ((ahome == null) || (phome == null)) {
	    useBeans("ebasic", true);
	    try {
		ahome = (AccountHome) PortableRemoteObject.narrow(
                                        ictx.lookup(BEAN_HOME_ACCOUNT),
					AccountHome.class);
		phome = (PersonHome) PortableRemoteObject.narrow(
                                        ictx.lookup(BEAN_HOME_PERSON),
					PersonHome.class);
	    } catch (NamingException e) {
		fail("Cannot get bean home: "+e.getMessage());
	    }
	}
    }

    /**
     * Creation of a bean with PK that maps a single field
     */
    public void testSpkCreate() throws Exception {
	int num = 1000;
	String customer = "Gertrude";

	Account acc = ahome.create(num, customer);
	Assert.assertEquals("Number", num, acc.getNumber());
	Assert.assertEquals("Customer", customer, acc.getCustomer());

	// cleaning
	acc.remove();
	
    }

    /**
     * Remove of a bean with PK that maps a single field
     */
    public void testSpkRemove() throws Exception {
	int num = 0;
	Account acc = null;

	ahome.remove(new Integer(num));
	try {
	    acc = ahome.findByNumber(num);
	    fail("bean already exists after a remove !!");
	} catch (FinderException e) {
	}

	// cleaning
	ahome.create(num, "to be removed");
	
    }

    /**
     * findByPrimaryKey of a bean with PK that maps a single field
     */
    public void testSpkFindByPrimaryKey() throws Exception {
	int num = 10;

	Account acc = ahome.findByPrimaryKey(new Integer(num));
	Assert.assertEquals("Number", num, acc.getNumber());
	
    }

    /**
     * findAll beans with PK that maps a single field
     */
    public void testSpkFindAll() throws Exception {
	Collection col = ahome.findAll();
    }

    /**
     * Creation of a bean with unknown PK class at the bean development phase
     */
    public void testUpkCreate() throws Exception {
	int num = 1000;
	String name = "Gertrude";

	Person prs = phome.create(num, name);
	Assert.assertEquals("Number", num, prs.getNumber());
	Assert.assertEquals("Customer", name, prs.getName());

	// cleaning
	prs.remove();
	
    }

    /**
     * Remove of a bean with unknown PK class at the bean development phase
     */
    public void testUpkRemove() throws Exception {
	int num = 0;
	Person prs = null;

	phome.remove(new Integer(num));
	try {
	    prs = phome.findByNumber(num);
	    fail("bean already exists after a remove !!");
	} catch (FinderException e) {
	}

	// cleaning
	phome.create(num, "to be removed");
	
    }

    /**
     * findByPrimaryKey of a bean with unknown PK class at the bean development phase
     */
    public void testUpkFindByPrimaryKey() throws Exception {
	int num = 10;

	Person prs = phome.findByPrimaryKey(new Integer(num));
	Assert.assertEquals("Number", num, prs.getNumber());
	
    }

    /**
     * findAll beans with unknown PK class at the bean development phase
     */
    public void testUpkFindAll() throws Exception {
	Collection col = phome.findAll();
    }

    public static Test suite() {
	return new TestSuite(F_VariousPK.class);
    }

    public static void main (String args[]) {
	String testtorun = null;
	// Get args
	for (int argn = 0; argn < args.length; argn++) {
	    String s_arg = args[argn];
	    Integer i_arg;
	    if (s_arg.equals("-n")) {
		testtorun = args[++argn];
	    }
	}
	if (testtorun == null) {
	    junit.textui.TestRunner.run(suite());
	} else {
	    junit.textui.TestRunner.run(new F_VariousPK(testtorun));
	}
    }
}
